# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
Formelly known as directory.mozilla.org.

DMOZ uses a hierarchical ontology scheme for organizing site listings.
Listings on a similar topic are grouped into categories
which can then include smaller categories.
"""

from gluish.benchmark import timed
from gluish.common import Executable
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi

class DMOZTask(DefaultTask):
    """ DMOZ supertask. """
    TAG = 'dmoz'

    def closest(self):
        return monthly(date=self.date)

class DMOZStructureDownload(DMOZTask):
    """ Download structure RDF. """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default='http://rdf.dmoz.org/rdf/structure.rdf.u8.gz', significant=False)

    def requires(self):
        return [Executable(name='curl'), Executable(name='gunzip')]

    @timed
    def run(self):
        output = shellout("""curl "{url}" | gunzip -c > {output}""", url=self.url)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='rdf'))

class DMOZContentDownload(DMOZTask):
    """ Download content RDF. """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default='http://rdf.dmoz.org/rdf/content.rdf.u8.gz', significant=False)

    def requires(self):
        return [Executable(name='curl'), Executable(name='gunzip')]

    @timed
    def run(self):
        output = shellout("""curl "{url}" | gunzip -c > {output}""", url=self.url)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='rdf'))
